package org.jeecg.modules.ctop.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.modules.ctop.entity.*;
import org.jeecg.modules.ctop.service.*;
import org.jeecg.modules.system.entity.SysDictItem;
import org.jeecg.modules.system.service.ISysDictItemService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;

/**
 * 头条广告主相关 2019年7月10日14:13:04
 *
 * @author 宋英豪
 */
@RestController
@RequestMapping("toutiao")
public class ByteDanceAdvertiserPostController {
    @Autowired
    private IByteDanceAdvertiserDataService advertiserDataService;

    /**
     * 1：获取广告主信息
     * @param accountId 本平台广告主id
     */
    @RequestMapping("/advertiser/info")
    public Map<String, Object> advertiserInfo(String accountId) {
        return advertiserDataService.getAdvertiserInfo(accountId);
    }

    /**
     * 2：获取广告主广告计划信息
     * @param accountId 本平台广告主id
     */
    @RequestMapping("/advertiser/plan/get")
    public Map<String, Object> advertiserPlan(String accountId, String ids) {
        return advertiserDataService.getAdvertiserPlan(accountId, ids);
    }

    /**
     * 3：修改广告计划信息状态
     * @param accountId 本平台广告主id
     */
    @RequestMapping("/advertiser/plan/update/status")
    public Map<String, Object> advertiserPlanUpdateStatus(String accountId, String adIds, String optStatus) {
        return advertiserDataService.advertiserPlanUpdateStatus(accountId, adIds, optStatus);
    }

    /**
     * 4：修改广告计划信息状态
     * @param accountId 本平台广告主id
     */
    @RequestMapping("/advertiser/plan/update/bid")
    public Map<String, Object> advertiserPlanUpdateBid(String accountId, String adIds, String bids) {
        return advertiserDataService.advertiserPlanUpdateBid(accountId, adIds, bids);
    }

    /**
     * 5：修改广告计划信息状态
     * @param accountId 本平台广告主id
     */
    @RequestMapping("/advertiser/plan/update/budget")
    public Map<String, Object> advertiserPlanUpdateBudget(String accountId, String adIds, String budgets) {
        return advertiserDataService.advertiserPlanUpdateBudget(accountId, adIds, budgets);
    }

    /**
     * 6：获取广告主广告预算信息
     * @param accountId 本平台广告主id
     */
    @RequestMapping("/advertiser/budget/get/")
    public Map<String, Object> advertiserBudget(String accountId) {
        return advertiserDataService.getAdvertiserBudget(accountId);
    }

    /**
     * 7：获取广告主广告组信息
     * @param accountId 本平台广告主id
     */
    @RequestMapping("/advertiser/campaign/get")
    public Map<String, Object> advertiserCampaign(String accountId, String ids) {
        return advertiserDataService.getAdvertiserCampaign(accountId, ids);
    }

    /**
     * 8：获取广告主广告创意信息
     * @param accountId 本平台广告主id
     */
    @RequestMapping("/advertiser/creative/get")
    public Map<String, Object> getAdvertiserCreative(String accountId, String ids) {
        return advertiserDataService.getAdvertiserCreative(accountId, ids);
    }

    /**
     * 9：修改广告创意信息状态
     * @param accountId 本平台广告主id
     */
    @RequestMapping("/advertiser/update/status/")
    public Map<String, Object> advertiserCreativeUpdateStatus(String accountId, String creativeIds, String optStatus) {
        return advertiserDataService.advertiserCreativeUpdateStatus(accountId, creativeIds, optStatus);
    }

    /**
     * 10：获取广告主广告创意素材信息
     * @param accountId 本平台广告主id
     * @param creativeIds 广告创意id
     */
    @RequestMapping("/advertiser/creative/material/read")
    public Map<String, Object> advertiserCreativeMaterial(String accountId, String creativeIds) {
        return advertiserDataService.getAdvertiserCreativeMaterial(accountId, creativeIds);
    }

    /**
     * 11：创建广告组信息
     * @param accountId 本平台广告主id
     */
//    @RequestMapping("/advertiser/campaign/create")
//    public Map<String, Object> advertiserCampaignCreate(String accountId, String campaignName, String budgetMode, Integer budget, String landingType) {
//        return advertiserDataService.advertiserCampaignCreate(accountId, campaignName, budgetMode, budget, landingType);
//    }

    /**
     * 11：创建广告计划信息
     */
    @RequestMapping("/advertiser/plan/create")
    public Map<String, Object> advertiserPlanCreate() {
        String accountId = "e9ca23d68d884d4ebb19d07889727dae";
        Long campaignId = 1640397758132260L;
        JSONObject data = new JSONObject();
        data.put("campaign_id", campaignId);
        data.put("delivery_range", "DEFAULT");
        data.put("budget_mode", "BUDGET_MODE_DAY");
        data.put("budget", 2000);//需要大于bid
        data.put("bid", 100);
        data.put("start_time", "2019-07-30 01:00");
        data.put("end_time", "2019-07-31 01:00");
        data.put("pricing", "PRICING_CPM");
        data.put("schedule_type", "SCHEDULE_FROM_NOW");
        data.put("flow_control_mode", "FLOW_CONTROL_MODE_FAST");
        data.put("name", "测试计划名称");
        data.put("external_url", "https://lordsmobile.176.com/download.php");
        data.put("download_url", "https://lordsmobile.176.com/download.php");
        data.put("download_type", "DOWNLOAD_URL");//坑
        data.put("package", "com.igg.android.lordsmobile_cn");//坑
        data.put("app_type", "APP_ANDROID");//坑
        return advertisePlanService.advertiserPlanCreate(new CTopOauthToken(), campaignId, data.toJSONString());
    }

    /**
     * 11：创建广告创意信息
     *
     * @param accountId 本平台广告主id
     */
    @RequestMapping("/advertiser/creative/create")
    public Map<String, Object> advertiserCreativeCreate(String accountId, Long campaignId, String dataString) {
        String getAccountId = "e9ca23d68d884d4ebb19d07889727dae";
        Long adId = 1640445647618076L;
        JSONObject data = new JSONObject();
        JSONArray jsonArray = new JSONArray();
        jsonArray.add("INVENTORY_FEED");
        jsonArray.add("INVENTORY_VIDEO_FEED");
        jsonArray.add("INVENTORY_HOTSOON_FEED");
        jsonArray.add("INVENTORY_AWEME_FEED");
        jsonArray.add("INVENTORY_UNION_SLOT");
        data.put("inventory_type", jsonArray);

        data.put("ad_id", adId);

        data.put("app_name", "asdasd");

        data.put("web_url", "http://www.chengzijianzhan.com/tetris/page/1593177139994627/");
        JSONArray keywords = new JSONArray();
        keywords.add("游戏");
        keywords.add("娱乐");
        keywords.add("塔防");
        keywords.add("养成");
        data.put("ad_keywords", keywords);
        JSONArray creatives = new JSONArray();
        JSONObject creative = new JSONObject();

        creative.put("image_mode", "CREATIVE_IMAGE_MODE_VIDEO_VERTICAL");
        creative.put("image_id", "71d10006f7067d0be514");
        creative.put("video_id", "v020334a0000bkvbakj6936tng75j6ag");
        creative.put("title", "测试标题11");
        creatives.add(creative);
        data.put("creatives", creatives);
        data.put("third_industry_id", 19050501);
        data.put("title", "测试创意标题123");
        return creativeService.creativeCreate(getAccountId, adId, data.toJSONString());
    }

    @Autowired
    private IByteDanceCreativeService creativeService;
    /**
     * 12：广告组更新状态信息 @param accountId 本平台广告主id
     */
    @RequestMapping("/advertiser/campaign/update/status")
    public Map<String, Object> advertiserCampaignUpdateStatus(String accountId, String campaignIds, String optStatus) {
        return advertiserDataService.advertiserCampaignUpdateStatus(accountId, campaignIds, optStatus);
    }

    /**
     * 13：广告组更新信息 @param accountId 本平台广告主id
     */
    @RequestMapping("/advertiser/campaign/update")
    public Map<String, Object> advertiserCampaignUpdate(String accountId, Long campaignId, String budgetMode, Integer budget, String campaignName) {
        return advertiserDataService.advertiserCampaignUpdate(accountId, campaignId, budgetMode, budget, campaignName);
    }

    /**
     * 14：获取人群包信息
     */
    @RequestMapping("/advertiser/custom/audience/select")
    public Map<String, Object> customAudienceSelect(String accountId) {
        return advertiserDataService.advertiserCustomAudienceSelect(accountId);
    }

    /**
     * 15：上传视频信息
     *
     * @param accountId 本平台广告主id
     * @
     */
    @RequestMapping("/advertiser/video/add")
    public Map<String, Object> advertiserVideoAdd(String accountId, String videoUrl) {
        return fileInfoService.uploadVideoToBytedance(accountId, videoUrl);
    }

    /**
     * 15：上传图片信息
     *
     * @param accountId 本平台广告主id
     * @
     */
    @RequestMapping("/advertiser/image/add")
    public Map<String, Object> advertiserImageAdd(String accountId, String imageUrl) {
        return fileInfoService.uploadImageToBytedance(accountId, imageUrl);
    }

    @RequestMapping("/industry/list")
    public Map<String, Object> industryList(String accountId, Integer level) {
        return fileInfoService.getIndustryList(accountId, level);
    }

    @Autowired
    private IFileInfoService fileInfoService;

    /**
     * @param audience
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     * @功能：查询字典数据
     */
    @RequestMapping(value = "audience/list", method = RequestMethod.GET)
    public Result<IPage<ByteDanceCustomAudience>> audiencePageList(ByteDanceCustomAudience audience, @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                   @RequestParam(name = "pageSize", defaultValue = "100") Integer pageSize, HttpServletRequest req) {
        Result<IPage<ByteDanceCustomAudience>> result = new Result<>();
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        audience.setAdvertiserId(user.getId());
        QueryWrapper<ByteDanceCustomAudience> queryWrapper = QueryGenerator.initQueryWrapper(audience, req.getParameterMap());
        queryWrapper.orderByAsc("sort_order");
        Page<ByteDanceCustomAudience> page = new Page<>(pageNo, pageSize);
        IPage<ByteDanceCustomAudience> pageList = audienceService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    @Autowired
    private IByteDanceCustomAudienceService audienceService;


    /**
     * @param sysDictItem
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     * @功能：查询字典数据
     */
    @RequestMapping(value = "dictitem/list", method = RequestMethod.GET)
    public Result<IPage<SysDictItem>> dictitemPageList(SysDictItem sysDictItem, @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                       @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize, HttpServletRequest req) {
        Result<IPage<SysDictItem>> result = new Result<IPage<SysDictItem>>();
        QueryWrapper<SysDictItem> queryWrapper = QueryGenerator.initQueryWrapper(sysDictItem, req.getParameterMap());
        queryWrapper.orderByAsc("sort_order");
        Page<SysDictItem> page = new Page<SysDictItem>(pageNo, pageSize);
        IPage<SysDictItem> pageList = sysDictItemService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    @Autowired
    private ISysDictItemService sysDictItemService;
    @Autowired
    private IByteDanceAdvertisePlanService advertisePlanService;
}
