package org.jeecg.modules.ctop.service.impl;

import cn.com.ctop.common.utils.PropertiesUtils;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.common.util.ResultMapUtils;
import org.jeecg.common.util.StatusCode;
import org.jeecg.modules.ctop.entity.ByteDanceCampaign;
import org.jeecg.modules.ctop.entity.ByteDanceCampaignTemplate;
import org.jeecg.modules.ctop.entity.CTopOauthToken;
import org.jeecg.modules.ctop.mapper.ByteDanceCampaignMapper;
import org.jeecg.modules.ctop.mapper.ByteDanceCampaignTemplateMapper;
import org.jeecg.modules.ctop.service.IByteDanceCampaignTemplateService;
import org.jeecg.modules.ctop.service.ICTopOauthTokenService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description: 今日头条广告组模板信息
 * @Author: jeecg-boot
 * @Date: 2019-07-22
 * @Version: V1.0
 */
@Service
public class ByteDanceCampaignTemplateServiceImpl extends ServiceImpl<ByteDanceCampaignTemplateMapper, ByteDanceCampaignTemplate> implements IByteDanceCampaignTemplateService {
    private static final Logger logger = LoggerFactory.getLogger(ByteDanceCampaignTemplateServiceImpl.class);
    @Override
    public Map<String, Object> insertTemplate(ByteDanceCampaignTemplate template) {
        Map<String, Object> resultMap = new HashMap<>();
        String name = template.getCampaignName();
        Long id = template.getId();
        if (null == name || "".equals(name)) {
            ResultMapUtils.setResultMap(resultMap, StatusCode.TEMPLATE_NAME_IS_NULL.getCode());
            return resultMap;
        }
        QueryWrapper<ByteDanceCampaignTemplate> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("campaign_name", template.getCampaignName());
        if (null != template.getId() && template.getId() != 0) {
            queryWrapper.ne("id", template.getId());
        }
        ByteDanceCampaignTemplate templateExist = getOne(queryWrapper);
        if (null != templateExist) {
            ResultMapUtils.setResultMap(resultMap, StatusCode.TEMPLATE_NAME_HAS_EXIST.getCode());
            return resultMap;
        }
        template.setStatus(1);
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        template.setAdvertiserId(user.getId());
        Date now = new Date();
        template.setCreateTime(now);
        template.setUpdateTime(now);
        this.saveOrUpdate(template);
        ResultMapUtils.setResultMap(resultMap, StatusCode.COMMON_SUCCESS.getCode());
        return resultMap;
    }
    @Override
    public Result<IPage<ByteDanceCampaignTemplate>> getList(ByteDanceCampaignTemplate template, Integer pageNo, Integer pageSize, HttpServletRequest req) {
        Result<IPage<ByteDanceCampaignTemplate>> result = new Result<>();
        QueryWrapper<ByteDanceCampaignTemplate> queryWrapper = QueryGenerator.initQueryWrapper(template, req.getParameterMap());
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        String advertiserId = user.getId();
        queryWrapper.eq("advertiser_id", advertiserId).orderByDesc("create_time");
        Page<ByteDanceCampaignTemplate> page = new Page<>(pageNo, pageSize);
        IPage<ByteDanceCampaignTemplate> pageList = this.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    @Override
    public Map<String, Object> campaignCreate(String accountId, String campaignName, String budgetMode, Integer budget, String landingType) {
        Map<String, Object> resultMap = new HashMap<>();
        CTopOauthToken token = tokenService.getOAuthTokenByAccountId(accountId);
        //2: 根据token以及用户id获取用户信息数据
        JSONObject params = new JSONObject();
        params.put("advertiser_id", token.getAccountId());
        params.put("campaign_name", campaignName);
        params.put("budget_mode", budgetMode);
        params.put("budget", budget);
        params.put("landing_type", landingType);
        JSONObject result = createCampaign(params, token.getAccessToken());
        Integer code = result.getInteger("code");

        if (null == code || !code.equals(0)) {
            logger.info("创建广告组接口异常==》accountId:{},message:{}", accountId, result.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "创建广告组接口异常");
            return resultMap;
        }
        JSONObject data = result.getJSONObject("data");
        if (null == data) {
            logger.info("广告组创建异常==》accountId:{},message:{}", accountId, result.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "广告组创建异常");
            return resultMap;
        }
        Long id = data.getLong("campaign_id");
        ByteDanceCampaign campaign = new ByteDanceCampaign(id, token, campaignName, budgetMode, budget, landingType);
        campaignMapper.insert(campaign);
        resultMap.put("code", 0);
        resultMap.put("message", "广告组创建成功");
        return resultMap;
    }

    @Override
    public Map<String, Object> campaignCreate(CTopOauthToken token, ByteDanceCampaignTemplate template) {
        Map<String, Object> resultMap = new HashMap<>();
        //2: 根据token以及用户id获取用户信息数据
        JSONObject params = new JSONObject();
        params.put("advertiser_id", token.getAccountId());
        params.put("campaign_name", template.getCampaignName());
        params.put("budget_mode", template.getCampaignBudget());
        params.put("budget", template.getBudgetDaily());
        params.put("landing_type", template.getPromotionPurpose());
        JSONObject result = createCampaign(params, token.getAccessToken());
        Integer code = result.getInteger("code");

        if (null == code || !code.equals(0)) {
            logger.info("创建广告组接口异常==》accountId:{},message:{}", token.getAccountId(), result.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", result.getString("message"));
            return resultMap;
        }
        JSONObject data = result.getJSONObject("data");
        if (null == data) {
            logger.info("广告组创建异常==》accountId:{},message:{}", token.getAccountId(), result.getString("message"));
            resultMap.put("code", -1);
            resultMap.put("message", "广告组创建异常");
            return resultMap;
        }
        Long id = data.getLong("campaign_id");
        ByteDanceCampaign campaign = new ByteDanceCampaign(id, token, template);
        //清除清数据，插入新数据
        campaignMapper.insert(campaign);
        resultMap.put("code", 0);
        resultMap.put("message", "广告组创建成功");
        return resultMap;
    }

    @Override
    public Map<String, Object> campaignCreate(String getAccountId, Long campaignId, String name) {
        ByteDanceCampaignTemplate template = campaignTemplateMapper.selectById(campaignId);
        CTopOauthToken token = tokenService.getOAuthTokenByAccountId(getAccountId);
        Map<String, Object> resultMap = new HashMap<>();
        //2: 根据token以及用户id获取用户信息数据
        JSONObject params = new JSONObject();
        params.put("advertiser_id", token.getAccountId());
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd_hh_mm_ss");
        params.put("campaign_name", name + "_广告组_" + sdf.format(new Date()) + "汇创思拓_" + System.currentTimeMillis());
        params.put("budget_mode", template.getCampaignBudget());
        params.put("budget", template.getBudgetDaily());
        params.put("landing_type", template.getPromotionPurpose());
        JSONObject result = createCampaign(params, token.getAccessToken());
        Integer code = result.getInteger("code");

        if (null == code || !code.equals(0)) {
            logger.info("创建广告组接口异常==》accountId:{},message:{}", token.getAccountId(), result.getString("message"));
            resultMap.put("success", false);
            resultMap.put("code", -1);
            resultMap.put("message", result.getString("message"));
            return resultMap;
        }
        JSONObject data = result.getJSONObject("data");
        if (null == data) {
            logger.info("广告组创建异常==》accountId:{},message:{}", token.getAccountId(), result.getString("message"));
            resultMap.put("success", false);
            resultMap.put("code", -1);
            resultMap.put("message", "广告组创建异常");
            return resultMap;
        }
        Long id = data.getLong("campaign_id");
        ByteDanceCampaign campaign = new ByteDanceCampaign(id, token, template);
        //清除清数据，插入新数据
        campaignMapper.insert(campaign);
        resultMap.put("campaignId", id);
        resultMap.put("success", true);
        resultMap.put("code", 0);
        resultMap.put("message", "广告组创建成功");
        return resultMap;
    }

    public JSONObject createCampaign(JSONObject data, String token) {
        // 请求地址
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_campaign_create");
        // 构造请求
        HttpPost httpEntity = new HttpPost(url);

        httpEntity.setHeader("Access-Token", token);

        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;

        try {
            client = HttpClientBuilder.create().build();
            httpEntity.setEntity(new StringEntity(data.toJSONString(), ContentType.APPLICATION_JSON));

            response = client.execute(httpEntity);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                return JSONObject.parseObject(result.toString());
            }

        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                client.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    @Autowired
    private ByteDanceCampaignTemplateMapper campaignTemplateMapper;
    @Autowired
    private ICTopOauthTokenService tokenService;
    @Autowired
    private ByteDanceCampaignMapper campaignMapper;
}
