package org.jeecg.modules.ctop.service.impl;

import cn.com.ctop.common.utils.PropertiesUtils;
import cn.com.ctop.toutiao.common.BytedanceInterfaceConstant;
import com.alibaba.druid.support.spring.stat.annotation.Stat;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.io.FileUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpEntityEnclosingRequestBase;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.entity.mime.content.FileBody;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.ResultMapUtils;
import org.jeecg.common.util.StatusCode;
import org.jeecg.modules.ctop.entity.*;
import org.jeecg.modules.ctop.mapper.ByteDanceImageInfoMapper;
import org.jeecg.modules.ctop.mapper.ByteDanceVideoInfoMapper;
import org.jeecg.modules.ctop.mapper.FileInfoMapper;
import org.jeecg.modules.ctop.service.IByteDanceImageInfoService;
import org.jeecg.modules.ctop.service.ICTopOauthTokenService;
import org.jeecg.modules.ctop.service.IFileInfoService;
import org.jeecg.modules.system.entity.SysCategory;
import org.jeecg.modules.system.mapper.SysCategoryMapper;
import org.jeecg.modules.system.service.ISysCategoryService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.FileSystemResource;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URI;
import java.net.URL;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description: 文件信息
 * @Author: jeecg-boot
 * @Date: 2019-07-28
 * @Version: V1.0
 */
@Service
public class FileInfoServiceImpl extends ServiceImpl<FileInfoMapper, FileInfo> implements IFileInfoService {
    private static final Logger logger = LoggerFactory.getLogger(FileInfoServiceImpl.class);
    private FileInfo fileInfo;

    @Override
    public Map<String, Object> uploadVideoToBytedance(String accountId, String videoUrl) {
        Map<String, Object> resultMap = new HashMap<>();
        //TODO查询是否已经上传过头条平台
        CTopOauthToken token = tokenService.getOAuthTokenByAccountId(accountId);
        JSONObject resultObject = uploadAdvideo(token, videoUrl, token.getAccountId() + "");
        System.out.println(resultObject);
        Integer code = resultObject.getInteger("code");
        String message = resultObject.getString("message");
        if (null == code || code != 0) {
            logger.info("视频文件上传失败,accountId:{},message:{}", accountId, message);
            ResultMapUtils.setResultMap(resultMap, StatusCode.BYTEDANCE_VIDEO_UPLOAD_FAIL.getCode());
            return resultMap;
        }
        fileInfo = (FileInfo) resultObject.get("file");

        ResultMapUtils.setResultMap(resultMap, StatusCode.COMMON_SUCCESS.getCode());
        resultMap.put("videoId", fileInfo.getFileId());
        return resultMap;
    }

    @Override
    public Map<String, Object> upload3ImagesToBytedance(String accountId, String imageFileUrls) {
        Map<String, Object> resultMap = new HashMap<>();
        String[] imageUrls = imageFileUrls.split(",");
        if (null == imageUrls || imageUrls.length < 3) {
            ResultMapUtils.setResultMap(resultMap, StatusCode.IMAGE_NUMBER_SHORTAGE.getCode());
            return resultMap;
        }
        CTopOauthToken token = tokenService.getOAuthTokenByAccountId(accountId);
        JSONArray imageIds = new JSONArray();
        for (int i = 0; i < imageUrls.length; i++) {
            JSONObject resultObject = uploadAdImage(token, imageUrls[i], token.getAccountId() + "");
            System.out.println(resultObject);
            Integer code = resultObject.getInteger("code");
            String message = resultObject.getString("message");
            if (null == code || code != 0) {
                logger.info("图片文件上传失败,accountId:{},message:{}", accountId, message);
                ResultMapUtils.setResultMap(resultMap, StatusCode.BYTEDANCE_VIDEO_UPLOAD_FAIL.getCode());
                return resultMap;
            }
            FileInfo fileInfo = (FileInfo) resultObject.get("file");
            imageIds.add(fileInfo.getFileId());
        }
        ResultMapUtils.setResultMap(resultMap, StatusCode.COMMON_SUCCESS.getCode());
        resultMap.put("imageIds", imageIds);
        return resultMap;
    }

    @Autowired
    private ByteDanceVideoInfoMapper videoInfoMapper;
    @Autowired
    private ByteDanceImageInfoMapper imageInfoMapper;

    @Override
    public Map<String, Object> uploadImageToBytedance(String accountId, String imageUrl) {
        Map<String, Object> resultMap = new HashMap<>();
        CTopOauthToken token = tokenService.getOAuthTokenByAccountId(accountId);
        JSONObject resultObject = uploadAdImage(token, imageUrl, token.getAccountId() + "");
        Integer code = resultObject.getInteger("code");
        String message = resultObject.getString("message");
        if (null == code || code != 0) {
            logger.info("图片文件上传失败,accountId:{},message:{}", accountId, message);
            ResultMapUtils.setResultMap(resultMap, StatusCode.BYTEDANCE_IMAGE_UPLOAD_FAIL.getCode());
            return resultMap;
        }
        FileInfo fileInfo = (FileInfo) resultObject.get("file");
        ResultMapUtils.setResultMap(resultMap, StatusCode.COMMON_SUCCESS.getCode());
        JSONArray array = new JSONArray();
        array.add(fileInfo.getFileId());
        resultMap.put("imageIds", array);
        return resultMap;
    }

    @Autowired
    private IByteDanceImageInfoService imageInfoService;
    @Autowired
    FileInfoMapper fileInfoMapper;
    @Override
    public Map<String, Object> getIndustryList(String accountId, Integer level) {
        Map<String, Object> resultMap = new HashMap<>();
        CTopOauthToken token = tokenService.getOAuthTokenByAccountId(accountId);
        JSONObject result = searchIndustry(token, level);
        JSONArray list = result.getJSONObject("data").getJSONArray("list");
        for (int i = 0; i < list.size(); i++) {
            JSONObject object = list.getJSONObject(i);
            SysCategory category = new SysCategory();
            String firstCode = object.getInteger("second_industry_id") + "";
            QueryWrapper<SysCategory> wrapper = new QueryWrapper();
            wrapper.eq("code", firstCode);
            SysCategory parent = sysCategoryMapper.selectOne(wrapper);
            category.setPid(parent.getId());
            category.setName(object.getString("third_industry_name"));
            category.setCode(object.getInteger("third_industry_id") + "");
            category.setCreateBy("admin");
            category.setCreateTime(new Date());
            category.setSysOrgCode("A01");
            sysCategoryMapper.insert(category);
        }
        resultMap.put("code", 0);
        resultMap.put("message", "数据获取成功");
        resultMap.put("data", result);
        return resultMap;
    }

    @Override
    public JSONArray getByteDanceIndustryList(HttpServletRequest req) {
        JSONArray result = new JSONArray();
        String pid = "6c6b0ad4f49b65f3943528e8a35d74fc";
        List<SysCategory> categories = categoryService.getListByPid(pid);
        for (SysCategory category : categories) {
            JSONObject object = new JSONObject();
            object.put("value", category.getCode());
            object.put("label", category.getName());
            JSONArray childListArray = new JSONArray();
            List<SysCategory> childList = categoryService.getListByPid(category.getId());
            for (SysCategory child : childList) {
                JSONObject childObject = new JSONObject();
                childObject.put("value", child.getCode());
                childObject.put("label", child.getName());
                JSONArray secendChildListArray = new JSONArray();
                List<SysCategory> secendChildList = categoryService.getListByPid(child.getId());
                for (SysCategory secendChild : secendChildList) {
                    JSONObject secendChildObject = new JSONObject();
                    secendChildObject.put("value", secendChild.getCode());
                    secendChildObject.put("label", secendChild.getName());
                    secendChildListArray.add(secendChildObject);
                }
                childObject.put("children", secendChildListArray);
                childListArray.add(childObject);
            }
            object.put("children", childListArray);
            result.add(object);
        }
        return result;
    }

    @Override
    public List<FileInfo> getFileInfoByMd5(String md5) {
        QueryWrapper<FileInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("md5", md5).orderByDesc("create_time");
        return fileInfoMapper.selectList(queryWrapper);
    }


    @Autowired
    private ISysCategoryService categoryService;

    @Autowired
    private SysCategoryMapper sysCategoryMapper;

    public JSONObject searchIndustry(CTopOauthToken token, Integer level) {
        // 请求地址
        String url = "https://ad.toutiao.com/open_api/2/tools/industry/get/";
        if (null != level) {
            url += "?level=" + level;
        }
        // 构造请求
        HttpEntityEnclosingRequestBase httpEntity = new HttpEntityEnclosingRequestBase() {
            @Override
            public String getMethod() {
                return "GET";
            }
        };
        httpEntity.setHeader("Access-Token", token.getAccessToken());
        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;
        try {
            client = HttpClientBuilder.create().build();
            httpEntity.setURI(URI.create(url));
            httpEntity.setEntity(new StringEntity(new JSONObject().toJSONString(), ContentType.APPLICATION_JSON));
            response = client.execute(httpEntity);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                return JSONObject.parseObject(result.toString());
            }
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                client.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    public JSONObject uploadAdImage(CTopOauthToken token, String imageUrl, String advertiserId) {
        JSONObject result = new JSONObject();
        //1:下载文件到本地
        String imagePath = downLoadByURL(imageUrl);
        //2: 获取MD5值
        String md5Hex = null;
        try {
            md5Hex = DigestUtils.md5Hex(new FileInputStream(imagePath));
        } catch (IOException e) {
            e.printStackTrace();
        }
        //3: 根据MD5值查询是否已经上传过服务器
        List<FileInfo> getFileInfo = getFileInfoByMd5(md5Hex);
        if (null != getFileInfo && getFileInfo.size() > 0) {//表示已经上传过服务器
            result.put("code", 0);
            result.put("file", getFileInfo.get(0));
            result.put("message", "图片上传成功");
            result.put("success", true);
            return result;
        }
        QueryWrapper<FileInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("md5", md5Hex);
        fileInfoMapper.delete(queryWrapper);

        //文件上传
        JSONObject bytedanceObject = imageUpload(token, advertiserId, imageUrl);
        if (null == bytedanceObject) {
            logger.info("今日头条图片上传失败");
            result.put("code", -1);
            result.put("message", "今日头条文件上传失败");
            return result;
        }
        JSONObject data = bytedanceObject.getJSONObject("data");
        FileInfo fileInfo = new FileInfo(md5Hex, data, advertiserId, token, "https:" + imageUrl, imagePath, "IMAGE", "1");
        fileInfoMapper.insert(fileInfo);
        result.put("code", 0);
        result.put("file", fileInfo);
        result.put("message", "图片上传成功");
        result.put("success", true);
        return result;
    }

    private JSONObject imageUpload(CTopOauthToken token, String advertiserId, String imageUrl) {
        JSONObject result = null;
        // 请求地址
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_file_image_ad");
        // 构造请求
        HttpPost httpPost = new HttpPost(url);
        httpPost.setHeader("Access-Token", token.getAccessToken());
        MultipartEntityBuilder entityBuilder = MultipartEntityBuilder.create();
        // 其他参数
        entityBuilder.addTextBody("advertiser_id", advertiserId);
        entityBuilder.addTextBody("upload_type", BytedanceInterfaceConstant.UPLOAD_TYPE_BY_URL);
        entityBuilder.addTextBody("image_url", "https:" + imageUrl);

        HttpEntity entity = entityBuilder.build();
        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;
        try {
            client = HttpClientBuilder.create().build();
            httpPost.setURI(URI.create(url));
            httpPost.setEntity(entity);
            response = client.execute(httpPost);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer buffer = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    buffer.append(line);
                }
                bufferedReader.close();
                return JSONObject.parseObject(buffer.toString());
            }
        } catch (ClientProtocolException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                client.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    @Value("${jeecg.path.image-upload}")
    private String imageUploadPath;
    @Value("${jeecg.path.video-upload}")
    private String videoUploadPath;

    public String downLoadByURL(String fileUrl) {
        //获取文件名，文件名实际上在URL中可以找到
        String fileName = fileUrl.substring(fileUrl.lastIndexOf("/") + 1);
        //这里服务器上要将此图保存的路径
        String savePath = this.imageUploadPath + fileName;
        try {
            URL url = new URL("https:" + fileUrl);/*将网络资源地址传给,即赋值给url*/
            /*此为联系获得网络资源的固定格式用法，以便后面的in变量获得url截取网络资源的输入流*/
            HttpURLConnection connection = (HttpURLConnection) url.openConnection();
            DataInputStream in = new DataInputStream(connection.getInputStream());
            /*此处也可用BufferedInputStream与BufferedOutputStream*/
            DataOutputStream out = new DataOutputStream(new FileOutputStream(savePath));
            /*将参数savePath，即将截取的图片的存储在本地地址赋值给out输出流所指定的地址*/
            byte[] buffer = new byte[4096];
            int count = 0;
            /*将输入流以字节的形式读取并写入buffer中*/
            while ((count = in.read(buffer)) > 0) {
                out.write(buffer, 0, count);
            }
            out.close();/*后面三行为关闭输入输出流以及网络资源的固定格式*/
            in.close();
            connection.disconnect();
            //返回内容是保存后的完整的URL
            return savePath;/*网络资源截取并存储本地成功返回true*/

        } catch (Exception e) {
            System.out.println(e + fileUrl + savePath);
            return null;
        }
    }

    public JSONObject uploadAdvideo(CTopOauthToken token, String videoUrl, String advertiserId) {
        JSONObject result = new JSONObject();
        //1:下载文件到本地
        String videoPath = downLoadByURL(videoUrl);
        //2: 获取MD5值
        String md5Hex = null;
        try {
            md5Hex = DigestUtils.md5Hex(new FileInputStream(videoPath));
        } catch (IOException e) {
            e.printStackTrace();
        }
        //3: 根据MD5值查询是否已经上传过服务器
        List<FileInfo> getFileInfo = getFileInfoByMd5(md5Hex);
        if (null != getFileInfo && getFileInfo.size() > 0) {//表示已经上传过服务器
            result.put("code", 0);
            result.put("file", getFileInfo.get(0));
            result.put("message", "视频上传成功");
            result.put("success", true);
            return result;
        }

        QueryWrapper<FileInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("md5", md5Hex);
        fileInfoMapper.delete(queryWrapper);

        //文件上传
        JSONObject bytedanceObject = videoUpload(token, advertiserId, videoPath);
        if (null == bytedanceObject) {
            logger.info("今日头条视频上传失败");
            result.put("code", -1);
            result.put("message", "今日头条视频上传失败");
            return result;
        }
        JSONObject data = bytedanceObject.getJSONObject("data");
        FileInfo fileInfo = new FileInfo(md5Hex, data, advertiserId, token, "https:" + videoUrl, videoPath, "VIDEO", "1");
        fileInfoMapper.insert(fileInfo);
        result.put("code", 0);
        result.put("file", fileInfo);
        result.put("message", "视频上传成功");
        result.put("success", true);
        return result;

    }

    private JSONObject videoUpload(CTopOauthToken token, String advertiserId, String videoPath) {
        CloseableHttpResponse response = null;
        CloseableHttpClient client = null;
        // 请求地址
        String url = PropertiesUtils.getValue("bytedance_config", "bytedance_api_url") + PropertiesUtils.getValue("bytedance_config", "bytedance_v2_file_video_ad");
        // 构造请求
        HttpPost httpPost = new HttpPost(url);
        httpPost.setHeader("Access-Token", token.getAccessToken());
        // 文件参数
        try {

            FileBody file = new FileBody(new File(videoPath));
            MultipartEntityBuilder entityBuilder = MultipartEntityBuilder.create()
                    .addPart("video_file", file);
            // 其他参数
            entityBuilder.addTextBody("advertiser_id", advertiserId);
            entityBuilder.addTextBody("video_signature", DigestUtils.md5Hex(new FileInputStream(new File(videoPath))));
            HttpEntity entity = entityBuilder.build();
            client = HttpClientBuilder.create().build();
            httpPost.setURI(URI.create(url));
            httpPost.setEntity(entity);
            response = client.execute(httpPost);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
                StringBuffer result = new StringBuffer();
                String line = "";
                while ((line = bufferedReader.readLine()) != null) {
                    result.append(line);
                }
                bufferedReader.close();
                return JSONObject.parseObject(result.toString());
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (response != null) {
                    response.close();
                }
                client.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    @Autowired
    private ICTopOauthTokenService tokenService;

    public static void main(String[] args) throws Exception {

//        FileUtils.copyURLToFile(new URL("https://ctop-media.oss-cn-beijing.aliyuncs.com/%E9%AB%98%E6%B8%85%E7%89%882-1565344662650.mp4"),new File("D://1234.mp4"));
//        File file = new File("https://ctop-media.oss-cn-beijing.aliyuncs.com/%E9%AB%98%E6%B8%85%E7%89%882-1565344662650.mp4");
//        FileInputStream fis = new FileInputStream(file);
//        fis.read();
    }
}
